// ---------- MAPI Types -----------

// all types found in MAPI TNEF attachment
var MAPI_UNSPECIFIED = 0x0000; // MAPI Unspecified
var MAPI_NULL = 0x0001; // MAPI null property
var MAPI_SHORT = 0x0002; // MAPI short (signed 16 bits)
var MAPI_INT = 0x0003; // MAPI integer (signed 32 bits)
var MAPI_FLOAT = 0x0004; // MAPI float (4 bytes)
var MAPI_DOUBLE = 0x0005; // MAPI double
var MAPI_CURRENCY = 0x0006; // MAPI currency (64 bits)
var MAPI_APPTIME = 0x0007; // MAPI application time
var MAPI_ERROR = 0x000a; // MAPI error (32 bits)
var MAPI_BOOLEAN = 0x000b; // MAPI boolean (16 bits)
var MAPI_OBJECT = 0x000d; // MAPI embedded object
var MAPI_INT64 = 0x0014; // MAPI 8 byte signed int
var MAPI_STRING = 0x001e; // MAPI string
var MAPI_UNICODE_STRING = 0x001f; // MAPI unicode-string (null terminated)
var MAPI_SYSTIME = 0x0040; // MAPI time (64 bits)
var MAPI_CLSID = 0x0048; // MAPI OLE GUID
var MAPI_BINARY = 0x0102; // MAPI binary
var MAPI_MULTI_STRING = 0x101F; // MAPI multiple string

function mattr_type_to_string(mattr_type) {
  switch (mattr_type) {
    case MAPI_UNSPECIFIED: return "UNSPECIFIED";
    case MAPI_NULL: return "NULL";
    case MAPI_SHORT: return "SHORT";
    case MAPI_INT: return "INT";
    case MAPI_FLOAT: return "FLOAT";
    case MAPI_DOUBLE: return "DOUBLE";
    case MAPI_CURRENCY: return "CURRENCY";
    case MAPI_APPTIME: return "APPTIME";
    case MAPI_ERROR: return "ERROR";
    case MAPI_BOOLEAN: return "BOOLEAN";
    case MAPI_OBJECT: return "OBJECT";
    case MAPI_INT64: return "INT64";
    case MAPI_STRING: return "STRING";
    case MAPI_UNICODE_STRING: return "UNICODE_STRING";
    case MAPI_SYSTIME: return "SYSTIME";
    case MAPI_CLSID: return "CLSID";
    case MAPI_BINARY: return "BINARY";
    case MAPI_MULTI_STRING: return "MULTI_STRING";
    
    default: return `0000${mattr_type.toString(16)}`.slice(-4) //(to_hex(mattr_type, 2));
  }
}

// ---------- MAPI Names -----------

// names of some attributes found in TNEF file
var MAPI_ACKNOWLEDGEMENT_MODE = 0x0001;
var MAPI_TEMPLATE_DATA = 0x0001;
var MAPI_MEETING_ATTENDEE_CRITICAL_CHANGE = 0x0001;
var MAPI_ALTERNATE_RECEIPIANT_ALLOWED = 0x0002;
var MAPI_MEETING_WHERE = 0x0002;
var MAPI_AUTHORIZING_USERS = 0x0003;
var MAPI_MEETING_GLOBAL_OBJECT_ID = 0x0003;
var MAPI_AUTO_FORWARD_COMMENT = 0x0004;
var MAPI_SCRIPT_DATA = 0x0004;
var MAPI_MEETING_IS_SILENT = 0x0004;
var MAPI_AUTO_FORWARDED = 0x0005;
var MAPI_MEETING_IS_RECURRING = 0x0005;
var MAPI_CONTENT_CONFIDENTIALITY_ALGORITHM_ID = 0x0006;
var MAPI_MEETING_REQUIRED_ATTENDEES = 0x0006;
var MAPI_CONTENT_CORRELATOR = 0x0007;
var MAPI_MEETING_OPTIONAL_ATTENDEES = 0x0007;
var MAPI_CONTENT_IDENTIFIER = 0x0008;
var MAPI_MEETING_RESOURCE_ATTENDEES = 0x0008;
var MAPI_MEETING_DELEGATE_MAIL = 0x0009;
var MAPI_CONTENT_RETURN_REQUESTED = 0x000A;
var MAPI_MEETING_IS_EXCEPTION = 0x000A;
var MAPI_CONVERSATION_KEY = 0x000B;
var MAPI_MEETING_SINGLE_INVITE = 0x000B;
var MAPI_MEETING_TIME_ZONE = 0x000C;
var MAPI_CONVERSION_EITS = 0x000C;
var MAPI_DEFERRED_DELIVERY_TIME = 0x000F;
var MAPI_DELIVER_TIME = 0x0010;
var MAPI_DISCARD_REASON = 0x0011;
var MAPI_DISCLOSURE_OF_RECIPIENTS = 0x0012;
var MAPI_DL_EXPANSION_HISTORY = 0x0013;
var MAPI_MEETING_MONTH_INTERVAL = 0x0013;
var MAPI_DL_EXPANSION_PROHIBITED = 0x0014;
var MAPI_EXPIRY_TIME = 0x0015;
var MAPI_IMPLICIT_CONVERSION_PROHIBITED = 0x0016;
var MAPI_IMPORTANCE = 0x0017;
var MAPI_IPM_ID = 0x0018;
var MAPI_LATEST_DELIVERY_TIME = 0x0019;
var MAPI_MESSAGE_CLASS = 0x001A;
var MAPI_MEETING_OWNER_CRITICAL_CHANGE = 0x001A;
var MAPI_MESSAGE_DELIVERY_ID = 0x001B;
var MAPI_MEETING_CALENDAR_TYPE = 0x001C;
var MAPI_MEETING_ALL_ATTENDEES = 0x001D;
var MAPI_MESSAGE_SECURITY_LABEL = 0x001E;
var MAPI_OBSOLETED_IPMS = 0x001F;
var MAPI_ORIGINALLY_INTENDED_RECIPIENT_NAME = 0x0020;
var MAPI_ORIGINAL_EITS = 0x0021;
var MAPI_ORIGINATOR_CERTIFICATE = 0x0022;
var MAPI_ORIGINATOR_DELIVERY_REPORT_REQUESTED = 0x0023;
var MAPI_MEETING_CLEAN_GLOBAL_OBJECT_ID = 0x0023;
var MAPI_ORIGINATOR_RETURN_ADDRESS = 0x0024;
var MAPI_MEETING_MESSAGE_CLASS = 0x0024;
var MAPI_PARENT_KEY = 0x0025;
var MAPI_PRIORITY = 0x0026;
var MAPI_MEETING_TYPE = 0x0026;
var MAPI_MEETING_OLD_LOCATION = 0x0028;
var MAPI_READ_RECEIPT_REQUESTED = 0x0029;
var MAPI_RECIPIENT_REASSIGNMENT_PROHIBITED = 0x002B;
var MAPI_RELATED_IPMS = 0x002D;
var MAPI_ORIGINAL_SENSITIVITY = 0x002E;
var MAPI_REPORT_TAG = 0x0031;
var MAPI_SENSITIVITY = 0x0036;
var MAPI_SUBJECT = 0x0037;
var MAPI_CLIENT_SUBMIT_TIME = 0x0039;
var MAPI_SUBJECT_PREFIX = 0x003D;
var MAPI_RECEIVED_BY_NAME = 0x0040;
var MAPI_SENT_REPRESENTING_ENTRYID = 0x0041;
var MAPI_SENT_REPRESENTING_NAME = 0x0042;
var MAPI_RCVD_REPRESENTING_ENTRYID = 0x0043;
var MAPI_RCVD_REPRESENTING_NAME = 0x0044;
var MAPI_REPORT_ENTRYID = 0x0045;
var MAPI_READ_RECEIPT_ENTRYID = 0x0046;
var MAPI_MESSAGE_SUBMISSION_ID = 0x0047;
var MAPI_PROVIDER_SUBMIT_TIME = 0x0048;
var MAPI_ORIGINAL_SUBJECT = 0x0049;
var MAPI_DISC_VAL = 0x004A;
var MAPI_ORIG_MESSAGE_CLASS = 0x004B;
var MAPI_ORIGINAL_AUTHOR_ENTRYID = 0x004C;
var MAPI_ORIGINAL_AUTHOR_NAME = 0x004D;
var MAPI_MESSAGE_TO_ME = 0x0057;
var MAPI_MESSAGE_CC_ME = 0x0058;
var MAPI_ORIG_SENT_REPRESENTING_NAME = 0x005D;
var MAPI_ORIG_SENT_REPRESENTING_ENTRYID = 0x005E;
var MAPI_START_DATE = 0x0060;
var MAPI_END_DATE = 0x0061;
var MAPI_OWNER_APPT_ID = 0x0062;
var MAPI_RESPONSE_REQUESTED = 0x0063;
var MAPI_SENT_REPRESENTING_ADDR_TYPE = 0x0064;
var MAPI_SENT_REPRESENTING_EMAIL = 0x0065;
var MAPI_ORIG_SENT_REPRESENTING_EMAIL = 0x0069;
var MAPI_CONVERSATION_TOPIC = 0x0070;
var MAPI_CONVERSATION_INDEX = 0x0071;
var MAPI_RECEIVED_BY_ADDRTYPE = 0x0075;
var MAPI_RECEIVED_BY_EMAIL_ADDRESS = 0x0076;
var MAPI_TRANSPORT_MESSAGE_HEADERS = 0x007D;
var MAPI_TNEF_CORRELATION_KEY = 0x007F;
var MAPI_EXPLICIT_CONVERSION = 0x0C01;
var MAPI_MESSAGE_TOKEN = 0x0C03;
var MAPI_REPLY_REQUESTED = 0x0C17;
var MAPI_SENDER_NAME = 0x0C1A;
var MAPI_SENDER_SEARCH_KEY = 0x0C1D;
var MAPI_SENDER_EMAIL_ADDRESS = 0x0C1F;
var MAPI_DELETE_AFTER_SUBMIT = 0x0E01;
var MAPI_DISPLAY_BCC = 0x0E02;
var MAPI_DISPLAY_CC = 0x0E03;
var MAPI_DISPLAY_TO = 0x0E04;
var MAPI_PARENT_DISPLAY = 0x0E05;
var MAPI_MESSAGE_DELIVERY_TIME = 0x0E06;
var MAPI_MESSAGE_FLAGS = 0x0E07;
var MAPI_MESSAGE_SIZE = 0x0E08;
var MAPI_PARENT_ENTRYID = 0x0E09;
var MAPI_SENTMAIL_ENTRYID = 0x0E0A;
var MAPI_MESSAGE_RECIPIENTS = 0x0E12;
var MAPI_MESSAGE_ATTACHMENTS = 0x0E13;
var MAPI_SUBMIT_FLAGS = 0x0E14;
var MAPI_HASATTACH = 0x0E1B;
var MAPI_BODY_CRC = 0x0E1C;
var MAPI_NORMALIZED_SUBJECT = 0x0E1D;
var MAPI_RTF_IN_SYNC = 0x0E1F;
var MAPI_ATTACH_SIZE = 0x0E20;
var MAPI_ATTACH_NUM = 0x0E21;
var MAPI_PREPROCESS = 0x0E22;
var MAPI_PRIMARY_SEND_ACCOUNT = 0x0E28;
var MAPI_NEXT_SEND_ACCOUNT = 0x0E29;
var MAPI_ACCESS = 0x0FF4;
var MAPI_ROW_TYPE = 0x0FF5;
var MAPI_INSTANCE_KEY = 0x0FF6;
var MAPI_ACCESS_LEVEL = 0x0FF7;
var MAPI_MAPPING_SIGNATURE = 0x0FF8;
var MAPI_RECORD_TYPE = 0x0FF9;
var MAPI_MESSAGE_TYPE = 0x0FFA;
var MAPI_OBJECT_TYPE = 0x0FFE;
var MAPI_ENTRY_ID = 0x0FFF;
var MAPI_BODY = 0x1000;
var MAPI_REPORT_TEXT = 0x1001;
var MAPI_RTF_SYNC_BODY_CRC = 0x1006;
var MAPI_RTF_SYNC_BODY_COUNT = 0x1007;
var MAPI_RTF_SYNC_BODY_TAG = 0x1008;
var MAPI_RTF_COMPRESSED = 0x1009;
var MAPI_RTF_SYNC_PREFIX_COUNT = 0x1010;
var MAPI_RTF_SYNC_TRAILING_COUNT = 0x1011;
var MAPI_BODY_HTML = 0x1013;
var MAPI_SMTP_MESSAGE_ID = 0x1035;
var MAPI_CONFLICT_ITEMS = 0x1098;
var MAPI_URL_COMP_NAME = 0x10F3;
var MAPI_ATTRIBUTE_READ_ONLY = 0x10F6;
var MAPI_CATEGORIES = 0x2328;
var MAPI_DISPLAY_NAME = 0x3001;
var MAPI_EMAIL_ADDRESS = 0x3003;
var MAPI_CREATION_TIME = 0x3007;
var MAPI_MODIFICATION_TIME = 0x3008;
var MAPI_SEARCH_KEY = 0x300B;
var MAPI_STORE_SUPPORT_MASK = 0x340D;
var MAPI_MDB_PROVIDER = 0x3414;
var MAPI_CONTAINER_FLAGS = 0x3600;
var MAPI_FOLDER_TYPE = 0x3601;
var MAPI_CONTENT_COUNT = 0x3602;
var MAPI_CONTENT_UNREAD = 0x3603;
var MAPI_CONTAINER_HIERARCHY = 0x360E;
var MAPI_CONTAINER_CONTENTS = 0x360F;
var MAPI_FOLDER_ASSOCIATED_CONTENTS = 0x3610;
var MAPI_CONTAINER_CLASS = 0x3613;
var MAPI_CONTAINER_MODIFY_VERSION = 0x3614;
var MAPI_ATTACHMENT_X400_PARAMETERS = 0x3700;
var MAPI_ATTACH_DATA_OBJ = 0x3701;
var MAPI_ATTACH_ENCODING = 0x3702;
var MAPI_ATTACH_EXTENSION = 0x3703;
var MAPI_ATTACH_FILENAME = 0x3704;
var MAPI_ATTACH_METHOD = 0x3705;
var MAPI_ATTACH_LONG_FILENAME = 0x3707;
var MAPI_ATTACH_PATHNAME = 0x3708;
var MAPI_ATTACH_RENDERING = 0x3709;
var MAPI_ATTACH_TAG = 0x370A;
var MAPI_ATTACHMENT_RENDERING_POSITION = 0x370B;
var MAPI_ATTACH_TRANSPORT_NAME = 0x370C;
var MAPI_ATTACH_LONG_PATHNAME = 0x370D;
var MAPI_ATTACH_MIME_TAG = 0x370E;
var MAPI_ATTACH_ADDITIONAL_INFO = 0x370F;
var MAPI_ATTACH_MIME_SEQUENCE = 0x3710;
var MAPI_ATTACH_CONTENT_ID = 0x3712;
var MAPI_ATTACH_CONTENT_LOCATION = 0x3713;
var MAPI_ATTACH_FLAGS = 0x3714;
var MAPI_DISPLAY_TYPE = 0x3900;
var MAPI_TEMPLATEID = 0x3902;
var MAPI_SMTP_ADDRESS = 0x39FE;
var MAPI_ACCOUNT = 0x3A00;
var MAPI_CONTACT_CALLBACK_PHONE = 0x3A02;
var MAPI_GENERATION = 0x3A05;
var MAPI_GIVEN_NAME = 0x3A06;
var MAPI_CONTACT_BUSINESS_PHONE = 0x3A08;
var MAPI_CONTACT_HOME_PHONE = 0x3A09;
var MAPI_INITIALS = 0x3A0A;
var MAPI_KEYWORD = 0x3A0B;
var MAPI_LANGUAGE = 0x3A0C;
var MAPI_LOCATION = 0x3A0D;
var MAPI_SURNAME = 0x3A11;
var MAPI_COMPANY_NAME = 0x3A16;
var MAPI_TITLE = 0x3A17;
var MAPI_DEPARTMENT_NAME = 0x3A18;
var MAPI_CONTACT_PRIMARY_PHONE = 0x3A1A;
var MAPI_CONTACT_BUSINESS_PHONE_2 = 0x3A1B;
var MAPI_CONTACT_OTHER_PHONE = 0x3A1F;
var MAPI_CONTACT_MOBILE_PHONE = 0x3A08;
var MAPI_CONTACT_BUSINESS_FAX = 0x3A24;
var MAPI_COUNTRY = 0x3A26;
var MAPI_LOCALITY = 0x3A27;
var MAPI_STATE_PROVINCE = 0x3A28;
var MAPI_HOME_PHONE_2 = 0x3A2F;
var MAPI_CONTACT_SEND_RICH_INFO = 0x3A40;
var MAPI_CONTACT_MIDDLE_NAME = 0x3A44;
var MAPI_DISPLAY_NAME_PREFIX = 0x3A45;
var MAPI_CONTACT_COMPUTER_NETWORK = 0x3A49;
var MAPI_CONTACT_TDD_PHONE = 0x3A4B;
var MAPI_CONTACT_COMPANY_MAIN_PHONE = 0x3A57;
var MAPI_CONTACT_CHILDRENS_NAMES = 0x3A58;
var MAPI_INTERNET_CPID = 0x3FDE;
var MAPI_CONFLICT_ENTRY_ID = 0x3FF0;
var MAPI_MESSAGE_LOCALE_ID = 0x3FF1;
var MAPI_CREATOR_NAME = 0x3FF8;
var MAPI_CREATOR_ENTRY_ID = 0x3FF9;
var MAPI_LAST_MODIFIER_NAME = 0x3FFA;
var MAPI_LAST_MODIFIER_ENTRY_ID = 0x3FFB;
var MAPI_CODE_PAGE = 0x3FFD;
var MAPI_SENDER_FLAGS = 0x4019;
var MAPI_SENT_REPRESENTING_FLAGS = 0x401A;
var MAPI_SENDER_SIMPLE_DISPLAY_NAME = 0x4030;
var MAPI_SENT_REPRESENTING_SIMPLE_DISPLAY_NAME = 0x4031;
var MAPI_CREATOR_SIMPLE_DISPLAY_NAME = 0x4038;
var MAPI_LAST_MODIFER_SIMPLE_DISPLAY_NAME = 0x4039;
var MAPI_CONTENT_FILTER_SPAM_CONFIDENCE = 0x4076;
var MAPI_INET_MAIL_OVERRIDE_FORMAT = 0x5902;
var MAPI_MSG_EDITOR_FORMAT = 0x5909;
var MAPI_EXCEPTION_REPLACE_TIME = 0x7FF9;
var MAPI_ATTACHMENT_LINK_ID = 0x7FFA;
var MAPI_EXCEPTION_START_TIME = 0x7FFB;
var MAPI_EXCEPTION_END_TIME = 0x7FFC;
var MAPI_ATTACHMENT_FLAGS = 0x7FFD;
var MAPI_ATTACHMENT_HIDDEN = 0x7FFE;
var MAPI_ATTACHMENT_CONTACT_PHOTO = 0x7FFF;
var MAPI_CONTACT_FILE_UNDER = 0x8005;
var MAPI_CONTACT_LAST_AND_FIRST_NAME = 0x8017;
var MAPI_CONTACT_COMPANY_AND_FULL_NAME = 0x8018;
var MAPI_ADDRESS_BOOK_PROVIDER_EMAIL_LIST = 0x8028;
var MAPI_ADDRESS_BOOK_PROVIDER_ARRAY_TYPE = 0x8029;
var MAPI_CONTACT_EMAIL_1_FULL = 0x8080;
var MAPI_CONTACT_EMAIL_1_ADDRESS_TYPE = 0x8082;
var MAPI_CONTACT_EMAIL_1_ADDRESS = 0x8083;
var MAPI_CONTACT_EMAIL_1_DISPLAY_NAME = 0x8084;
var MAPI_CONTACT_EMAIL_1_ENTRY_ID = 0x8085;
var MAPI_CONTACT_EMAIL_2_FULL = 0x8090;
var MAPI_CONTACT_EMAIL_2_ADDRESS_TYPE = 0x8092;
var MAPI_CONTACT_EMAIL_2_ADDRESS = 0x8093;
var MAPI_CONTACT_EMAIL_2_DISPLAY_NAME = 0x8094;
var MAPI_CONTACT_EMAIL_2_ENTRY_ID = 0x8095;
var MAPI_CONTACT_COUNTRY_CODE = 0x80DD;
var MAPI_TASK_STATUS = 0x8101;
var MAPI_TASK_PERCENT_COMPLETE = 0x8102;
var MAPI_TASK_TEAM_TASK = 0x8103;
var MAPI_TASK_START_DATE = 0x8104;
var MAPI_TASK_DUE_DATE = 0x8105;
var MAPI_TASK_RESET_REMINDER = 0x8107;
var MAPI_TASK_ACCEPTED = 0x8108;
var MAPI_TASK_DEAD_OCCURRENCE = 0x8109;
var MAPI_TASK_DATE_COMPLETED = 0x810F;
var MAPI_TASK_ACTUAL_EFFORT = 0x8110;
var MAPI_TASK_ESTIMATED_EFFORT = 0x8111;
var MAPI_TASK_VERSION = 0x8112;
var MAPI_TASK_STATE = 0x8113;
var MAPI_TASK_LAST_UPDATE = 0x8115;
var MAPI_TASK_RECURRENCE = 0x8116;
var MAPI_TASK_ASSIGNERS = 0x8117;
var MAPI_TASK_STATUS_ON_COMPLETE = 0x8119;
var MAPI_TASK_HISTORY = 0x811A;
var MAPI_TASK_UPDATES = 0x811B;
var MAPI_TASK_COMPLETE = 0x811C;
var MAPI_TASK_F_CREATOR = 0x811E;
var MAPI_TASK_MULTIPLE_RECIPIENTS = 0X8120;
var MAPI_TASK_ASSIGNER = 0x8121;
var MAPI_TASK_LAST_USER = 0x8122;
var MAPI_TASK_ORDINAL = 0x8123;
var MAPI_TASK_NO_COMPUTE = 0x8124;
var MAPI_TASK_LAST_DELEGATE = 0x8125;
var MAPI_TASK_F_RECURRING = 0x8126;
var MAPI_TASK_ROLE = 0x8127;
var MAPI_TASK_OWNERSHIP = 0x8129;
var MAPI_TASK_ACCEPTANCE_STATE = 0x812A;
var MAPI_TASK_F_FIX_OFFLINE = 0x812C;
var MAPI_APPOINTMENT_SEND_AS_ICAL = 0x8200;
var MAPI_APPOINTMENT_SEQUENCE = 0x8201;
var MAPI_APPOINTMENT_SEQUENCE_TIME = 0x8202;
var MAPI_APPOINTMENT_LAST_SEQUENCE = 0x8203;
var MAPI_APPOINTMENT_CHANGE_HIGHLIGHT = 0x8204;
var MAPI_APPOINTMENT_BUSY_STATUS = 0x8205; // (0 = free, 1 = tentative, 2 = busy, 3 out of office)
var MAPI_APPOINTMENT_AUXILIARY_FLAGS = 0x8207;
var MAPI_APPOINTMENT_LOCATION = 0x8208;
var MAPI_APPOINTMENT_START_WHOLE = 0x820D;
var MAPI_APPOINTMENT_END_WHOLE = 0x820E;
var MAPI_APPOINTMENT_END_DATE = 0x8211;
var MAPI_APPOINTMENT_START_DATE = 0x8212;
var MAPI_APPOINTMENT_DURATION = 0x8213;
var MAPI_APPOINTMENT_COLOR = 0x8214; // (1 = red, 2 = blue, 3 = green, 4 = grey, 5 = orange, 6 = cyan, 7 = olive, 8 = purple, 9 = teal, a = yellow)
var MAPI_APPOINTMENT_ALL_DAY = 0x8215; // subtype
var MAPI_APPOINTMENT_RECUR = 0x8216;
var MAPI_APPOINTMENT_STATE_FLAGS = 0x8217; // (1 = M = meeting, 2 = R = received, 3 = C = canceled)
var MAPI_APPOINTMENT_RESPONSE_STATUS = 0x8218; // (0 = not req'd, 1 = organized, 2 = tentative, 3 = accepted, 4 = declined, 5 = not responded)
var MAPI_APPOINTMENT_REPLY_TIME = 0x8230;
var MAPI_APPOINTMENT_RECURRENCE_SET = 0x8223; // Recurring (0 not, 1 daily, 2 weekly, etc)
var MAPI_APPOINTMENT_INTENDED_BUSY_STATUS = 0x8224; // (0 = free, 1 = tentative, 2 = busy, 3 out of office)
var MAPI_APPOINTMENT_F_INVITED = 0x8229;
var MAPI_APPOINTMENT_REPLY_NAME = 0x8230;
var MAPI_APPOINTMENT_RECURRENCE_TYPE = 0x8231;
var MAPI_APPOINTMENT_RECURRENCE_PATTERN = 0x8232;
var MAPI_APPOINTMENT_RECURRENCE_TIMEZONE = 0x8234;
var MAPI_APPOINTMENT_RECURRENCE_START = 0x8235;
var MAPI_APPOINTMENT_RECURRENCE_END = 0x8236;
var MAPI_APPOINTMENT_ALL_ATTENDEES = 0x8238;
var MAPI_APPOINTMENT_AUTO_FILL_LOCATION = 0x823A;
var MAPI_APPOINTMENT_REQUIRED_ATTENDEES = 0x823B;
var MAPI_APPOINTMENT_OPTIONAL_ATTENDEES = 0x823C;
var MAPI_APPOINTMENT_UNSENDABLE_RECIPIENTS = 0x823D;
var MAPI_APPOINTMENT_CONFERENCING_CHECK = 0x8240;
var MAPI_APPOINTMENT_CONFERENCING_TYPE = 0x8241;
var MAPI_APPOINTMENT_DIRECTORY = 0x8242;
var MAPI_APPOINTMENT_ORGANIZER_ALIAS = 0x8243;
var MAPI_APPOINTMENT_AUTO_START_CHECK = 0x8244;
var MAPI_APPOINTMENT_AUTO_START_WHEN = 0x8245;
var MAPI_APPOINTMENT_ALLOW_EXTERNAL_CHECK = 0x8246;
var MAPI_APPOINTMENT_COLLABORATE_DOC = 0x8247;
var MAPI_APPOINTMENT_NET_SHOW_URL = 0x8248;
var MAPI_APPOINTMENT_ONLINE_PASSWORD = 0x8249;
var MAPI_APPOINTMENT_PROPOSED_START_WHOLE = 0x8250;
var MAPI_APPOINTMENT_PROPOSED_END_WHOLE = 0x8251;
var MAPI_APPOINTMENT_TZ_START_DISPLAY = 0x825E;
var MAPI_APPOINTMENT_TZ_END_DISPLAY = 0x825F;
var MAPI_APPOINTMENT_PROPOSED_DURATION = 0x8256;
var MAPI_APPOINTMENT_COUNTER_PROPOSAL = 0x8257;
var MAPI_APPOINTMENT_PROPOSAL_NUMBER = 0x8259;
var MAPI_APPOINTMENT_NOT_ALLOW_PROPOSE = 0x825A;
var MAPI_COMMON_REMINDER_MINUTES_BEFORE = 0x8501;
var MAPI_COMMON_REMINDER_DATE = 0x8502;
var MAPI_COMMON_REMINDER_SET = 0x8503;
var MAPI_COMMON_PRIVATE = 0x8506;
var MAPI_COMMON_AGING_DONT_AGE = 0x850E;
var MAPI_COMMON_ALLOW_EXCEPTION_DELETE = 0x8510;
var MAPI_COMMON_START = 0x8516;
var MAPI_COMMON_END = 0x8517;
var MAPI_TASK_MODE = 0x8518;
var MAPI_COMMON_REMINDER_OVERRIDE = 0x851C;
var MAPI_COMMON_REMINDER_TYPE = 0x851D;
var MAPI_COMMON_REMINDER_PLAY_SOUND = 0x851E;
var MAPI_COMMON_COMPANIES = 0x8539;
var MAPI_COMMON_CONTACTS = 0x853A;
var MAPI_NEXT_REMINDER_TIME = 0x8560; // Reminder date
var MAPI_USE_TNEF = 0x8582;
var MAPI_COMMON_CONTACT_LINK_NAME = 0x8586;
var MAPI_COMMON_CLASSIFIED = 0x85B5;
var MAPI_COMMON_CLASSIFICATION = 0x85B6;
var MAPI_COMMON_CLASSIFICATION_DESCRIPTION = 0x85B7;
var MAPI_COMMON_CLASSIFICATION_GUID = 0x85B8;
var MAPI_COMMON_CLASSIFICATION_KEEP = 0x85BA;


// convertion regex
//   from: const MAPI_\([^ \t]*\)[ \t].*$
//   to:   case MAPI_\1: return "\1";


function mattr_name_to_string(mattr_name, msg_class) {
  if (msg_class && (msg_class.substring(0, 15) == "IPM.Appointment" ||
    msg_class.substring(0, 12) == "IPM.Schedule" ||
    msg_class.substring(0, 22) == "IPM.Microsoft Schedule")) {
    switch (mattr_name) {
      case MAPI_MEETING_ATTENDEE_CRITICAL_CHANGE: return "MEETING_ATTENDEE_CRITICAL_CHANGE";
      case MAPI_MEETING_WHERE: return "MEETING_WHERE";
      case MAPI_MEETING_GLOBAL_OBJECT_ID: return "MEETING_GLOBAL_OBJECT_ID";
      case MAPI_MEETING_IS_SILENT: return "MEETING_IS_SILENT";
      case MAPI_MEETING_IS_RECURRING: return "MEETING_IS_RECURRING";
      case MAPI_MEETING_REQUIRED_ATTENDEES: return "MEETING_REQUIRED_ATTENDEES";
      case MAPI_MEETING_OPTIONAL_ATTENDEES: return "MEETING_OPTIONAL_ATTENDEES";
      case MAPI_MEETING_RESOURCE_ATTENDEES: return "MEETING_RESOURCE_ATTENDEES";
      case MAPI_MEETING_DELEGATE_MAIL: return "MEETING_DELEGATE_MAIL";
      case MAPI_MEETING_IS_EXCEPTION: return "MEETING_IS_EXCEPTION";
      case MAPI_MEETING_SINGLE_INVITE: return "MEETING_SINGLE_INVITE";
      case MAPI_MEETING_TIME_ZONE: return "MEETING_TIME_ZONE";
      case MAPI_MEETING_MONTH_INTERVAL: return "MEETING_MONTH_INTERVAL";
      case MAPI_MEETING_OWNER_CRITICAL_CHANGE: return "MEETING_OWNER_CRITICAL_CHANGE";
      case MAPI_MEETING_CALENDAR_TYPE: return "MEETING_CALENDAR_TYPE";
      case MAPI_MEETING_ALL_ATTENDEES: return "MEETING_ALL_ATTENDEES";
      case MAPI_MEETING_MESSAGE_CLASS: return "MEETING_MESSAGE_CLASS";
      case MAPI_MEETING_TYPE: return "MEETING_TYPE";
      case MAPI_MEETING_OLD_LOCATION: return "MEETING_OLD_LOCATION";
    }
  }

  switch (mattr_name) {
    case MAPI_ACKNOWLEDGEMENT_MODE: return "ACKNOWLEDGEMENT_MODE";
    case MAPI_TEMPLATE_DATA: return "TEMPLATE_DATA";
    case MAPI_MEETING_ATTENDEE_CRITICAL_CHANGE: return "MEETING_ATTENDEE_CRITICAL_CHANGE";
    case MAPI_ALTERNATE_RECEIPIANT_ALLOWED: return "ALTERNATE_RECEIPIANT_ALLOWED";
    case MAPI_MEETING_WHERE: return "MEETING_WHERE";
    case MAPI_AUTHORIZING_USERS: return "AUTHORIZING_USERS";
    case MAPI_MEETING_GLOBAL_OBJECT_ID: return "MEETING_GLOBAL_OBJECT_ID";
    case MAPI_AUTO_FORWARD_COMMENT: return "AUTO_FORWARD_COMMENT";
    case MAPI_SCRIPT_DATA: return "SCRIPT_DATA";
    case MAPI_MEETING_IS_SILENT: return "MEETING_IS_SILENT";
    case MAPI_AUTO_FORWARDED: return "AUTO_FORWARDED";
    case MAPI_MEETING_IS_RECURRING: return "MEETING_IS_RECURRING";
    case MAPI_CONTENT_CONFIDENTIALITY_ALGORITHM_ID: return "CONTENT_CONFIDENTIALITY_ALGORITHM_ID";
    case MAPI_MEETING_REQUIRED_ATTENDEES: return "MEETING_REQUIRED_ATTENDEES";
    case MAPI_CONTENT_CORRELATOR: return "CONTENT_CORRELATOR";
    case MAPI_MEETING_OPTIONAL_ATTENDEES: return "MEETING_OPTIONAL_ATTENDEES";
    case MAPI_CONTENT_IDENTIFIER: return "CONTENT_IDENTIFIER";
    case MAPI_MEETING_RESOURCE_ATTENDEES: return "MEETING_RESOURCE_ATTENDEES";
    case MAPI_MEETING_DELEGATE_MAIL: return "MEETING_DELEGATE_MAIL";
    case MAPI_CONTENT_RETURN_REQUESTED: return "CONTENT_RETURN_REQUESTED";
    case MAPI_MEETING_IS_EXCEPTION: return "MEETING_IS_EXCEPTION";
    case MAPI_CONVERSATION_KEY: return "CONVERSATION_KEY";
    case MAPI_MEETING_SINGLE_INVITE: return "MEETING_SINGLE_INVITE";
    case MAPI_MEETING_TIME_ZONE: return "MEETING_TIME_ZONE";
    case MAPI_CONVERSION_EITS: return "CONVERSION_EITS";
    case MAPI_DEFERRED_DELIVERY_TIME: return "DEFERRED_DELIVERY_TIME";
    case MAPI_DELIVER_TIME: return "DELIVER_TIME";
    case MAPI_DISCARD_REASON: return "DISCARD_REASON";
    case MAPI_DISCLOSURE_OF_RECIPIENTS: return "DISCLOSURE_OF_RECIPIENTS";
    case MAPI_DL_EXPANSION_HISTORY: return "DL_EXPANSION_HISTORY";
    case MAPI_MEETING_MONTH_INTERVAL: return "MEETING_MONTH_INTERVAL";
    case MAPI_DL_EXPANSION_PROHIBITED: return "DL_EXPANSION_PROHIBITED";
    case MAPI_EXPIRY_TIME: return "EXPIRY_TIME";
    case MAPI_IMPLICIT_CONVERSION_PROHIBITED: return "IMPLICIT_CONVERSION_PROHIBITED";
    case MAPI_IMPORTANCE: return "IMPORTANCE";
    case MAPI_IPM_ID: return "IPM_ID";
    case MAPI_LATEST_DELIVERY_TIME: return "LATEST_DELIVERY_TIME";
    case MAPI_MESSAGE_CLASS: return "MESSAGE_CLASS";
    case MAPI_MEETING_OWNER_CRITICAL_CHANGE: return "MEETING_OWNER_CRITICAL_CHANGE";
    case MAPI_MESSAGE_DELIVERY_ID: return "MESSAGE_DELIVERY_ID";
    case MAPI_MEETING_CALENDAR_TYPE: return "MEETING_CALENDAR_TYPE";
    case MAPI_MEETING_ALL_ATTENDEES: return "MEETING_ALL_ATTENDEES";
    case MAPI_MESSAGE_SECURITY_LABEL: return "MESSAGE_SECURITY_LABEL";
    case MAPI_OBSOLETED_IPMS: return "OBSOLETED_IPMS";
    case MAPI_ORIGINALLY_INTENDED_RECIPIENT_NAME: return "ORIGINALLY_INTENDED_RECIPIENT_NAME";
    case MAPI_ORIGINAL_EITS: return "ORIGINAL_EITS";
    case MAPI_ORIGINATOR_CERTIFICATE: return "ORIGINATOR_CERTIFICATE";
    case MAPI_ORIGINATOR_DELIVERY_REPORT_REQUESTED: return "ORIGINATOR_DELIVERY_REPORT_REQUESTED";
    case MAPI_MEETING_CLEAN_GLOBAL_OBJECT_ID: return "MEETING_CLEAN_GLOBAL_OBJECT_ID";
    case MAPI_ORIGINATOR_RETURN_ADDRESS: return "ORIGINATOR_RETURN_ADDRESS";
    case MAPI_MEETING_MESSAGE_CLASS: return "MEETING_MESSAGE_CLASS";
    case MAPI_PARENT_KEY: return "PARENT_KEY";
    case MAPI_PRIORITY: return "PRIORITY";
    case MAPI_MEETING_TYPE: return "MEETING_TYPE";
    case MAPI_MEETING_OLD_LOCATION: return "MEETING_OLD_LOCATION";
    case MAPI_READ_RECEIPT_REQUESTED: return "READ_RECEIPT_REQUESTED";
    case MAPI_RECIPIENT_REASSIGNMENT_PROHIBITED: return "RECIPIENT_REASSIGNMENT_PROHIBITED";
    case MAPI_RELATED_IPMS: return "RELATED_IPMS";
    case MAPI_ORIGINAL_SENSITIVITY: return "ORIGINAL_SENSITIVITY";
    case MAPI_REPORT_TAG: return "REPORT_TAG";
    case MAPI_SENSITIVITY: return "SENSITIVITY";
    case MAPI_SUBJECT: return "SUBJECT";
    case MAPI_CLIENT_SUBMIT_TIME: return "CLIENT_SUBMIT_TIME";
    case MAPI_SUBJECT_PREFIX: return "SUBJECT_PREFIX";
    case MAPI_RECEIVED_BY_NAME: return "RECEIVED_BY_NAME";
    case MAPI_SENT_REPRESENTING_ENTRYID: return "SENT_REPRESENTING_ENTRYID";
    case MAPI_SENT_REPRESENTING_NAME: return "SENT_REPRESENTING_NAME";
    case MAPI_RCVD_REPRESENTING_ENTRYID: return "RCVD_REPRESENTING_ENTRYID";
    case MAPI_RCVD_REPRESENTING_NAME: return "RCVD_REPRESENTING_NAME";
    case MAPI_REPORT_ENTRYID: return "REPORT_ENTRYID";
    case MAPI_READ_RECEIPT_ENTRYID: return "READ_RECEIPT_ENTRYID";
    case MAPI_MESSAGE_SUBMISSION_ID: return "MESSAGE_SUBMISSION_ID";
    case MAPI_PROVIDER_SUBMIT_TIME: return "PROVIDER_SUBMIT_TIME";
    case MAPI_ORIGINAL_SUBJECT: return "ORIGINAL_SUBJECT";
    case MAPI_DISC_VAL: return "DISC_VAL";
    case MAPI_ORIG_MESSAGE_CLASS: return "ORIG_MESSAGE_CLASS";
    case MAPI_ORIGINAL_AUTHOR_ENTRYID: return "ORIGINAL_AUTHOR_ENTRYID";
    case MAPI_ORIGINAL_AUTHOR_NAME: return "ORIGINAL_AUTHOR_NAME";
    case MAPI_MESSAGE_TO_ME: return "MESSAGE_TO_ME";
    case MAPI_MESSAGE_CC_ME: return "MESSAGE_CC_ME";
    case MAPI_START_DATE: return "START_DATE";
    case MAPI_END_DATE: return "END_DATE";
    case MAPI_OWNER_APPT_ID: return "OWNER_APPT_ID";
    case MAPI_RESPONSE_REQUESTED: return "RESPONSE_REQUESTED";
    case MAPI_SENT_REPRESENTING_ADDR_TYPE: return "SENT_REPRESENTING_ADDR_TYPE";
    case MAPI_SENT_REPRESENTING_EMAIL: return "SENT_REPRESENTING_EMAIL";
    case MAPI_CONVERSATION_TOPIC: return "CONVERSATION_TOPIC";
    case MAPI_CONVERSATION_INDEX: return "CONVERSATION_INDEX";
    case MAPI_RECEIVED_BY_ADDRTYPE: return "RECEIVED_BY_ADDRTYPE";
    case MAPI_RECEIVED_BY_EMAIL_ADDRESS: return "RECEIVED_BY_EMAIL_ADDRESS";
    case MAPI_TRANSPORT_MESSAGE_HEADERS: return "TRANSPORT_MESSAGE_HEADERS";
    case MAPI_TNEF_CORRELATION_KEY: return "TNEF_CORRELATION_KEY";
    case MAPI_EXPLICIT_CONVERSION: return "EXPLICIT_CONVERSION";
    case MAPI_MESSAGE_TOKEN: return "MESSAGE_TOKEN";
    case MAPI_REPLY_REQUESTED: return "REPLY_REQUESTED";
    case MAPI_SENDER_NAME: return "SENDER_NAME";
    case MAPI_SENDER_SEARCH_KEY: return "SENDER_SEARCH_KEY";
    case MAPI_SENDER_EMAIL_ADDRESS: return "SENDER_EMAIL_ADDRESS";
    case MAPI_DELETE_AFTER_SUBMIT: return "DELETE_AFTER_SUBMIT";
    case MAPI_DISPLAY_BCC: return "DISPLAY_BCC";
    case MAPI_DISPLAY_CC: return "DISPLAY_CC";
    case MAPI_DISPLAY_TO: return "DISPLAY_TO";
    case MAPI_PARENT_DISPLAY: return "PARENT_DISPLAY";
    case MAPI_MESSAGE_DELIVERY_TIME: return "MESSAGE_DELIVERY_TIME";
    case MAPI_MESSAGE_FLAGS: return "MESSAGE_FLAGS";
    case MAPI_MESSAGE_SIZE: return "MESSAGE_SIZE";
    case MAPI_PARENT_ENTRYID: return "PARENT_ENTRYID";
    case MAPI_SENTMAIL_ENTRYID: return "SENTMAIL_ENTRYID";
    case MAPI_MESSAGE_RECIPIENTS: return "MESSAGE_RECIPIENTS";
    case MAPI_MESSAGE_ATTACHMENTS: return "MESSAGE_ATTACHMENTS";
    case MAPI_SUBMIT_FLAGS: return "SUBMIT_FLAGS";
    case MAPI_HASATTACH: return "HASATTACH";
    case MAPI_BODY_CRC: return "BODY_CRC";
    case MAPI_NORMALIZED_SUBJECT: return "NORMALIZED_SUBJECT";
    case MAPI_RTF_IN_SYNC: return "RTF_IN_SYNC";
    case MAPI_ATTACH_SIZE: return "ATTACH_SIZE";
    case MAPI_ATTACH_NUM: return "ATTACH_NUM";
    case MAPI_PREPROCESS: return "PREPROCESS";
    case MAPI_PRIMARY_SEND_ACCOUNT: return "PRIMARY_SEND_ACCOUNT";
    case MAPI_NEXT_SEND_ACCOUNT: return "NEXT_SEND_ACCOUNT";
    case MAPI_ACCESS: return "ACCESS";
    case MAPI_ROW_TYPE: return "ROW_TYPE";
    case MAPI_INSTANCE_KEY: return "INSTANCE_KEY";
    case MAPI_ACCESS_LEVEL: return "ACCESS_LEVEL";
    case MAPI_MAPPING_SIGNATURE: return "MAPPING_SIGNATURE";
    case MAPI_RECORD_TYPE: return "RECORD_TYPE";
    case MAPI_MESSAGE_TYPE: return "MESSAGE_TYPE";
    case MAPI_OBJECT_TYPE: return "OBJECT_TYPE";
    case MAPI_ENTRY_ID: return "ENTRY_ID";
    case MAPI_BODY: return "BODY";
    case MAPI_REPORT_TEXT: return "REPORT_TEXT";
    case MAPI_RTF_SYNC_BODY_CRC: return "RTF_SYNC_BODY_CRC";
    case MAPI_RTF_SYNC_BODY_COUNT: return "RTF_SYNC_BODY_COUNT";
    case MAPI_RTF_SYNC_BODY_TAG: return "RTF_SYNC_BODY_TAG";
    case MAPI_RTF_COMPRESSED: return "RTF_COMPRESSED";
    case MAPI_RTF_SYNC_PREFIX_COUNT: return "RTF_SYNC_PREFIX_COUNT";
    case MAPI_RTF_SYNC_TRAILING_COUNT: return "RTF_SYNC_TRAILING_COUNT";
    case MAPI_BODY_HTML: return "BODY_HTML";
    case MAPI_SMTP_MESSAGE_ID: return "SMTP_MESSAGE_ID";
    case MAPI_CONFLICT_ITEMS: return "CONFLICT_ITEMS";
    case MAPI_URL_COMP_NAME: return "URL_COMP_NAME";
    case MAPI_ATTRIBUTE_READ_ONLY: return "ATTRIBUTE_READ_ONLY";
    case MAPI_CATEGORIES: return "CATEGORIES";
    case MAPI_DISPLAY_NAME: return "DISPLAY_NAME";
    case MAPI_EMAIL_ADDRESS: return "EMAIL_ADDRESS";
    case MAPI_CREATION_TIME: return "CREATION_TIME";
    case MAPI_MODIFICATION_TIME: return "MODIFICATION_TIME";
    case MAPI_SEARCH_KEY: return "SEARCH_KEY";
    case MAPI_STORE_SUPPORT_MASK: return "STORE_SUPPORT_MASK";
    case MAPI_MDB_PROVIDER: return "MDB_PROVIDER";
    case MAPI_CONTAINER_FLAGS: return "CONTAINER_FLAGS";
    case MAPI_FOLDER_TYPE: return "FOLDER_TYPE";
    case MAPI_CONTENT_COUNT: return "CONTENT_COUNT";
    case MAPI_CONTENT_UNREAD: return "CONTENT_UNREAD";
    case MAPI_CONTAINER_HIERARCHY: return "CONTAINER_HIERARCHY";
    case MAPI_CONTAINER_CONTENTS: return "CONTAINER_CONTENTS";
    case MAPI_FOLDER_ASSOCIATED_CONTENTS: return "FOLDER_ASSOCIATED_CONTENTS";
    case MAPI_CONTAINER_CLASS: return "CONTAINER_CLASS";
    case MAPI_CONTAINER_MODIFY_VERSION: return "CONTAINER_MODIFY_VERSION";
    case MAPI_ATTACHMENT_X400_PARAMETERS: return "ATTACHMENT_X400_PARAMETERS";
    case MAPI_ATTACH_DATA_OBJ: return "ATTACH_DATA_OBJ";
    case MAPI_ATTACH_ENCODING: return "ATTACH_ENCODING";
    case MAPI_ATTACH_EXTENSION: return "ATTACH_EXTENSION";
    case MAPI_ATTACH_FILENAME: return "ATTACH_FILENAME";
    case MAPI_ATTACH_METHOD: return "ATTACH_METHOD";
    case MAPI_ATTACH_LONG_FILENAME: return "ATTACH_LONG_FILENAME";
    case MAPI_ATTACH_PATHNAME: return "ATTACH_PATHNAME";
    case MAPI_ATTACH_RENDERING: return "ATTACH_RENDERING";
    case MAPI_ATTACH_TAG: return "ATTACH_TAG";
    case MAPI_ATTACHMENT_RENDERING_POSITION: return "ATTACHMENT_RENDERING_POSITION";
    case MAPI_ATTACH_TRANSPORT_NAME: return "ATTACH_TRANSPORT_NAME";
    case MAPI_ATTACH_LONG_PATHNAME: return "ATTACH_LONG_PATHNAME";
    case MAPI_ATTACH_MIME_TAG: return "ATTACH_MIME_TAG";
    case MAPI_ATTACH_ADDITIONAL_INFO: return "ATTACH_ADDITIONAL_INFO";
    case MAPI_ATTACH_MIME_SEQUENCE: return "ATTACH_MIME_SEQUENCE";
    case MAPI_ATTACH_CONTENT_ID: return "ATTACH_CONTENT_ID";
    case MAPI_ATTACH_CONTENT_LOCATION: return "ATTACH_CONTENT_LOCATION";
    case MAPI_ATTACH_FLAGS: return "ATTACH_FLAGS";
    case MAPI_DISPLAY_TYPE: return "DISPLAY_TYPE";
    case MAPI_TEMPLATEID: return "TEMPLATEID";
    case MAPI_ACCOUNT: return "ACCOUNT";
    case MAPI_CONTACT_CALLBACK_PHONE: return "CONTACT_CALLBACK_PHONE";
    case MAPI_GENERATION: return "GENERATION";
    case MAPI_GIVEN_NAME: return "GIVEN_NAME";
    case MAPI_CONTACT_BUSINESS_PHONE: return "CONTACT_BUSINESS_PHONE";
    case MAPI_CONTACT_HOME_PHONE: return "CONTACT_HOME_PHONE";
    case MAPI_INITIALS: return "INITIALS";
    case MAPI_KEYWORD: return "KEYWORD";
    case MAPI_LANGUAGE: return "LANGUAGE";
    case MAPI_LOCATION: return "LOCATION";
    case MAPI_SURNAME: return "SURNAME";
    case MAPI_COMPANY_NAME: return "COMPANY_NAME";
    case MAPI_TITLE: return "TITLE";
    case MAPI_DEPARTMENT_NAME: return "DEPARTMENT_NAME";
    case MAPI_CONTACT_PRIMARY_PHONE: return "CONTACT_PRIMARY_PHONE";
    case MAPI_CONTACT_BUSINESS_PHONE_2: return "CONTACT_BUSINESS_PHONE_2";
    case MAPI_CONTACT_OTHER_PHONE: return "CONTACT_OTHER_PHONE";
    case MAPI_CONTACT_MOBILE_PHONE: return "CONTACT_MOBILE_PHONE";
    case MAPI_CONTACT_BUSINESS_FAX: return "CONTACT_BUSINESS_FAX";
    case MAPI_COUNTRY: return "COUNTRY";
    case MAPI_LOCALITY: return "LOCALITY";
    case MAPI_STATE_PROVINCE: return "STATE_PROVINCE";
    case MAPI_HOME_PHONE_2: return "HOME_PHONE_2";
    case MAPI_CONTACT_SEND_RICH_INFO: return "CONTACT_SEND_RICH_INFO";
    case MAPI_CONTACT_MIDDLE_NAME: return "CONTACT_MIDDLE_NAME";
    case MAPI_DISPLAY_NAME_PREFIX: return "DISPLAY_NAME_PREFIX";
    case MAPI_CONTACT_COMPUTER_NETWORK: return "CONTACT_COMPUTER_NETWORK";
    case MAPI_CONTACT_TDD_PHONE: return "CONTACT_TDD_PHONE";
    case MAPI_CONTACT_COMPANY_MAIN_PHONE: return "CONTACT_COMPANY_MAIN_PHONE";
    case MAPI_CONTACT_CHILDRENS_NAMES: return "CONTACT_CHILDRENS_NAMES";
    case MAPI_INTERNET_CPID: return "INTERNET_CPID";
    case MAPI_CONFLICT_ENTRY_ID: return "CONFLICT_ENTRY_ID";
    case MAPI_MESSAGE_LOCALE_ID: return "MESSAGE_LOCALE_ID";
    case MAPI_CREATOR_NAME: return "CREATOR_NAME";
    case MAPI_CREATOR_ENTRY_ID: return "CREATOR_ENTRY_ID";
    case MAPI_LAST_MODIFIER_NAME: return "LAST_MODIFIER_NAME";
    case MAPI_LAST_MODIFIER_ENTRY_ID: return "LAST_MODIFIER_ENTRY_ID";
    case MAPI_CODE_PAGE: return "CODE_PAGE";
    case MAPI_SENDER_FLAGS: return "SENDER_FLAGS";
    case MAPI_SENT_REPRESENTING_FLAGS: return "SENT_REPRESENTING_FLAGS";
    case MAPI_SENDER_SIMPLE_DISPLAY_NAME: return "SENDER_SIMPLE_DISPLAY_NAME";
    case MAPI_SENT_REPRESENTING_SIMPLE_DISPLAY_NAME: return "SENT_REPRESENTING_SIMPLE_DISPLAY_NAME";
    case MAPI_CREATOR_SIMPLE_DISPLAY_NAME: return "CREATOR_SIMPLE_DISPLAY_NAME";
    case MAPI_LAST_MODIFER_SIMPLE_DISPLAY_NAME: return "LAST_MODIFER_SIMPLE_DISPLAY_NAME";
    case MAPI_CONTENT_FILTER_SPAM_CONFIDENCE: return "CONTENT_FILTER_SPAM_CONFIDENCE";
    case MAPI_INET_MAIL_OVERRIDE_FORMAT: return "INET_MAIL_OVERRIDE_FORMAT";
    case MAPI_MSG_EDITOR_FORMAT: return "MSG_EDITOR_FORMAT";
    case MAPI_EXCEPTION_REPLACE_TIME: return "EXCEPTION_REPLACE_TIME";
    case MAPI_ATTACHMENT_LINK_ID: return "ATTACHMENT_LINK_ID";
    case MAPI_EXCEPTION_START_TIME: return "EXCEPTION_START_TIME";
    case MAPI_EXCEPTION_END_TIME: return "EXCEPTION_END_TIME";
    case MAPI_ATTACHMENT_FLAGS: return "ATTACHMENT_FLAGS";
    case MAPI_ATTACHMENT_HIDDEN: return "ATTACHMENT_HIDDEN";
    case MAPI_ATTACHMENT_CONTACT_PHOTO: return "ATTACHMENT_CONTACT_PHOTO";
    case MAPI_CONTACT_FILE_UNDER: return "CONTACT_FILE_UNDER";
    case MAPI_CONTACT_LAST_AND_FIRST_NAME: return "CONTACT_LAST_AND_FIRST_NAME";
    case MAPI_CONTACT_COMPANY_AND_FULL_NAME: return "CONTACT_COMPANY_AND_FULL_NAME";
    case MAPI_ADDRESS_BOOK_PROVIDER_EMAIL_LIST: return "ADDRESS_BOOK_PROVIDER_EMAIL_LIST";
    case MAPI_ADDRESS_BOOK_PROVIDER_ARRAY_TYPE: return "ADDRESS_BOOK_PROVIDER_ARRAY_TYPE";
    case MAPI_CONTACT_EMAIL_1_FULL: return "CONTACT_EMAIL_1_FULL";
    case MAPI_CONTACT_EMAIL_1_ADDRESS_TYPE: return "CONTACT_EMAIL_1_ADDRESS_TYPE";
    case MAPI_CONTACT_EMAIL_1_ADDRESS: return "CONTACT_EMAIL_1_ADDRESS";
    case MAPI_CONTACT_EMAIL_1_DISPLAY_NAME: return "CONTACT_EMAIL_1_DISPLAY_NAME";
    case MAPI_CONTACT_EMAIL_1_ENTRY_ID: return "CONTACT_EMAIL_1_ENTRY_ID";
    case MAPI_CONTACT_EMAIL_2_FULL: return "CONTACT_EMAIL_2_FULL";
    case MAPI_CONTACT_EMAIL_2_ADDRESS_TYPE: return "CONTACT_EMAIL_2_ADDRESS_TYPE";
    case MAPI_CONTACT_EMAIL_2_ADDRESS: return "CONTACT_EMAIL_2_ADDRESS";
    case MAPI_CONTACT_EMAIL_2_DISPLAY_NAME: return "CONTACT_EMAIL_2_DISPLAY_NAME";
    case MAPI_CONTACT_EMAIL_2_ENTRY_ID: return "CONTACT_EMAIL_2_ENTRY_ID";
    case MAPI_CONTACT_COUNTRY_CODE: return "CONTACT_COUNTRY_CODE";
    case MAPI_TASK_STATUS: return "TASK_STATUS";
    case MAPI_TASK_PERCENT_COMPLETE: return "TASK_PERCENT_COMPLETE";
    case MAPI_TASK_TEAM_TASK: return "TASK_TEAM_TASK";
    case MAPI_TASK_START_DATE: return "TASK_START_DATE";
    case MAPI_TASK_DUE_DATE: return "TASK_DUE_DATE";
    case MAPI_TASK_RESET_REMINDER: return "TASK_RESET_REMINDER";
    case MAPI_TASK_ACCEPTED: return "TASK_ACCEPTED";
    case MAPI_TASK_DEAD_OCCURRENCE: return "TASK_DEAD_OCCURRENCE";
    case MAPI_TASK_DATE_COMPLETED: return "TASK_DATE_COMPLETED";
    case MAPI_TASK_ACTUAL_EFFORT: return "TASK_ACTUAL_EFFORT";
    case MAPI_TASK_ESTIMATED_EFFORT: return "TASK_ESTIMATED_EFFORT";
    case MAPI_TASK_VERSION: return "TASK_VERSION";
    case MAPI_TASK_STATE: return "TASK_STATE";
    case MAPI_TASK_LAST_UPDATE: return "TASK_LAST_UPDATE";
    case MAPI_TASK_RECURRENCE: return "TASK_RECURRENCE";
    case MAPI_TASK_ASSIGNERS: return "TASK_ASSIGNERS";
    case MAPI_TASK_STATUS_ON_COMPLETE: return "TASK_STATUS_ON_COMPLETE";
    case MAPI_TASK_HISTORY: return "TASK_HISTORY";
    case MAPI_TASK_UPDATES: return "TASK_UPDATES";
    case MAPI_TASK_COMPLETE: return "TASK_COMPLETE";
    case MAPI_TASK_F_CREATOR: return "TASK_F_CREATOR";
    case MAPI_TASK_MULTIPLE_RECIPIENTS: return "TASK_MULTIPLE_RECIPIENTS";
    case MAPI_TASK_ASSIGNER: return "TASK_ASSIGNER";
    case MAPI_TASK_LAST_USER: return "TASK_LAST_USER";
    case MAPI_TASK_ORDINAL: return "TASK_ORDINAL";
    case MAPI_TASK_NO_COMPUTE: return "TASK_NO_COMPUTE";
    case MAPI_TASK_LAST_DELEGATE: return "TASK_LAST_DELEGATE";
    case MAPI_TASK_F_RECURRING: return "TASK_F_RECURRING";
    case MAPI_TASK_ROLE: return "TASK_ROLE";
    case MAPI_TASK_OWNERSHIP: return "TASK_OWNERSHIP";
    case MAPI_TASK_ACCEPTANCE_STATE: return "TASK_ACCEPTANCE_STATE";
    case MAPI_TASK_F_FIX_OFFLINE: return "TASK_F_FIX_OFFLINE";
    case MAPI_APPOINTMENT_SEND_AS_ICAL: return "APPOINTMENT_SEND_AS_ICAL";
    case MAPI_APPOINTMENT_SEQUENCE: return "APPOINTMENT_SEQUENCE";
    case MAPI_APPOINTMENT_SEQUENCE_TIME: return "APPOINTMENT_SEQUENCE_TIME";
    case MAPI_APPOINTMENT_LAST_SEQUENCE: return "APPOINTMENT_LAST_SEQUENCE";
    case MAPI_APPOINTMENT_CHANGE_HIGHLIGHT: return "APPOINTMENT_CHANGE_HIGHLIGHT";
    case MAPI_APPOINTMENT_BUSY_STATUS: return "APPOINTMENT_BUSY_STATUS";
    case MAPI_APPOINTMENT_AUXILIARY_FLAGS: return "APPOINTMENT_AUXILIARY_FLAGS";
    case MAPI_APPOINTMENT_LOCATION: return "APPOINTMENT_LOCATION";
    case MAPI_APPOINTMENT_START_WHOLE: return "APPOINTMENT_START_WHOLE";
    case MAPI_APPOINTMENT_END_WHOLE: return "APPOINTMENT_END_WHOLE";
    case MAPI_APPOINTMENT_END_DATE: return "APPOINTMENT_END_DATE";
    case MAPI_APPOINTMENT_START_DATE: return "APPOINTMENT_START_DATE";
    case MAPI_APPOINTMENT_DURATION: return "APPOINTMENT_DURATION";
    case MAPI_APPOINTMENT_COLOR: return "APPOINTMENT_COLOR";
    case MAPI_APPOINTMENT_ALL_DAY: return "APPOINTMENT_ALL_DAY";
    case MAPI_APPOINTMENT_RECUR: return "APPOINTMENT_RECUR";
    case MAPI_APPOINTMENT_STATE_FLAGS: return "APPOINTMENT_STATE_FLAGS";
    case MAPI_APPOINTMENT_RESPONSE_STATUS: return "APPOINTMENT_RESPONSE_STATUS";
    case MAPI_APPOINTMENT_REPLY_TIME: return "APPOINTMENT_REPLY_TIME";
    case MAPI_APPOINTMENT_RECURRENCE_SET: return "APPOINTMENT_RECURRENCE_SET";
    case MAPI_APPOINTMENT_INTENDED_BUSY_STATUS: return "APPOINTMENT_INTENDED_BUSY_STATUS";
    case MAPI_APPOINTMENT_F_INVITED: return "APPOINTMENT_F_INVITED";
    case MAPI_APPOINTMENT_REPLY_NAME: return "APPOINTMENT_REPLY_NAME";
    case MAPI_APPOINTMENT_RECURRENCE_TYPE: return "APPOINTMENT_RECURRENCE_TYPE";
    case MAPI_APPOINTMENT_RECURRENCE_PATTERN: return "APPOINTMENT_RECURRENCE_PATTERN";
    case MAPI_APPOINTMENT_RECURRENCE_TIMEZONE: return "APPOINTMENT_RECURRENCE_TIMEZONE";
    case MAPI_APPOINTMENT_RECURRENCE_START: return "APPOINTMENT_RECURRENCE_START";
    case MAPI_APPOINTMENT_RECURRENCE_END: return "APPOINTMENT_RECURRENCE_END";
    case MAPI_APPOINTMENT_ALL_ATTENDEES: return "APPOINTMENT_ALL_ATTENDEES";
    case MAPI_APPOINTMENT_AUTO_FILL_LOCATION: return "APPOINTMENT_AUTO_FILL_LOCATION";
    case MAPI_APPOINTMENT_REQUIRED_ATTENDEES: return "APPOINTMENT_REQUIRED_ATTENDEES";
    case MAPI_APPOINTMENT_OPTIONAL_ATTENDEES: return "APPOINTMENT_OPTIONAL_ATTENDEES";
    case MAPI_APPOINTMENT_UNSENDABLE_RECIPIENTS: return "APPOINTMENT_UNSENDABLE_RECIPIENTS";
    case MAPI_APPOINTMENT_CONFERENCING_CHECK: return "APPOINTMENT_CONFERENCING_CHECK";
    case MAPI_APPOINTMENT_CONFERENCING_TYPE: return "APPOINTMENT_CONFERENCING_TYPE";
    case MAPI_APPOINTMENT_DIRECTORY: return "APPOINTMENT_DIRECTORY";
    case MAPI_APPOINTMENT_ORGANIZER_ALIAS: return "APPOINTMENT_ORGANIZER_ALIAS";
    case MAPI_APPOINTMENT_AUTO_START_CHECK: return "APPOINTMENT_AUTO_START_CHECK";
    case MAPI_APPOINTMENT_AUTO_START_WHEN: return "APPOINTMENT_AUTO_START_WHEN";
    case MAPI_APPOINTMENT_ALLOW_EXTERNAL_CHECK: return "APPOINTMENT_ALLOW_EXTERNAL_CHECK";
    case MAPI_APPOINTMENT_COLLABORATE_DOC: return "APPOINTMENT_COLLABORATE_DOC";
    case MAPI_APPOINTMENT_NET_SHOW_URL: return "APPOINTMENT_NET_SHOW_URL";
    case MAPI_APPOINTMENT_ONLINE_PASSWORD: return "APPOINTMENT_ONLINE_PASSWORD";
    case MAPI_APPOINTMENT_PROPOSED_START_WHOLE: return "APPOINTMENT_PROPOSED_START_WHOLE";
    case MAPI_APPOINTMENT_PROPOSED_END_WHOLE: return "APPOINTMENT_PROPOSED_END_WHOLE";
    case MAPI_APPOINTMENT_TZ_START_DISPLAY: return "APPOINTMENT_TZ_START_DISPLAY";
    case MAPI_APPOINTMENT_TZ_END_DISPLAY: return "APPOINTMENT_TZ_END_DISPLAY";
    case MAPI_APPOINTMENT_PROPOSED_DURATION: return "APPOINTMENT_PROPOSED_DURATION";
    case MAPI_APPOINTMENT_COUNTER_PROPOSAL: return "APPOINTMENT_COUNTER_PROPOSAL";
    case MAPI_APPOINTMENT_PROPOSAL_NUMBER: return "APPOINTMENT_PROPOSAL_NUMBER";
    case MAPI_APPOINTMENT_NOT_ALLOW_PROPOSE: return "APPOINTMENT_NOT_ALLOW_PROPOSE";
    case MAPI_COMMON_REMINDER_MINUTES_BEFORE: return "COMMON_REMINDER_MINUTES_BEFORE";
    case MAPI_COMMON_REMINDER_DATE: return "COMMON_REMINDER_DATE";
    case MAPI_COMMON_REMINDER_SET: return "COMMON_REMINDER_SET";
    case MAPI_COMMON_PRIVATE: return "COMMON_PRIVATE";
    case MAPI_COMMON_AGING_DONT_AGE: return "COMMON_AGING_DONT_AGE";
    case MAPI_COMMON_ALLOW_EXCEPTION_DELETE: return "COMMON_ALLOW_EXCEPTION_DELETE";
    case MAPI_COMMON_START: return "COMMON_START";
    case MAPI_COMMON_END: return "COMMON_END";
    case MAPI_TASK_MODE: return "TASK_MODE";
    case MAPI_COMMON_REMINDER_OVERRIDE: return "COMMON_REMINDER_OVERRIDE";
    case MAPI_COMMON_REMINDER_TYPE: return "COMMON_REMINDER_TYPE";
    case MAPI_COMMON_REMINDER_PLAY_SOUND: return "COMMON_REMINDER_PLAY_SOUND";
    case MAPI_COMMON_COMPANIES: return "COMMON_COMPANIES";
    case MAPI_COMMON_CONTACTS: return "COMMON_CONTACTS";
    case MAPI_NEXT_REMINDER_TIME: return "NEXT_REMINDER_TIME";
    case MAPI_USE_TNEF: return "USE_TNEF";
    case MAPI_COMMON_CONTACT_LINK_NAME: return "COMMON_CONTACT_LINK_NAME";
    case MAPI_COMMON_CLASSIFIED: return "COMMON_CLASSIFIED";
    case MAPI_COMMON_CLASSIFICATION: return "COMMON_CLASSIFICATION";
    case MAPI_COMMON_CLASSIFICATION_DESCRIPTION: return "COMMON_CLASSIFICATION_DESCRIPTION";
    case MAPI_COMMON_CLASSIFICATION_GUID: return "COMMON_CLASSIFICATION_GUID";
    case MAPI_COMMON_CLASSIFICATION_KEEP: return "COMMON_CLASSIFICATION_KEEP";

    default: return `0000${mattr_name.toString(16)}`.slice(-4)  // (to_hex(mattr_name, 2));
  }
}

